------------------------------------------------------------------
--FCOGuildBankQuickSelect.lua
--Author: Baertram
--[[
Quickly select the guild bank/guild store by numbers 1-5 (at an opened guild bank/guild store selection)
]]
------------------------------------------------------------------

--Addon variables
local addonVars = {}
addonVars.gAddonName                 = "FCOGuildBankQuickSelect"
addonVars.addonNameMenu              = "FCO GuildBankQuickSelect"
addonVars.addonNameMenuDisplay       = "|c00FF00FCO |cFFFF00GuildBankQuickSelect|r"
addonVars.addonAuthor                = '|cFFFF00Baertram|r'
addonVars.addonVersion               = 0.01 -- Changing this will reset SavedVariables!
addonVars.addonVersionOptions        = '0.1.0' -- version shown in the settings panel
addonVars.addonSavedVariablesName    = "FCOGuildBankQuickSelect_Settings"
local DIALOG_PRIMARY
local DIALOG_NEGATIVE

--Control names of ZO* standard controls etc.
--Guild bank
local GUILDBANK_SELECT_POPUP                = ZO_SelectGuildBankDialog
local GUILDBANK_SELECT_POPUP_GUILD_COMBOBOX = ZO_SelectGuildBankDialogGuild.m_comboBox
local GUILDBANK_SELECT_POPUP_MODAL_UNDERLAY = ZO_SelectGuildBankDialogModalUnderlay
--Guild store
local GUILDSTORE_SELECT_POPUP                = ZO_SelectTradingHouseGuildDialog
local GUILDSTORE_SELECT_POPUP_GUILD_COMBOBOX = ZO_SelectTradingHouseGuildDialogGuild.m_comboBox
local GUILDSTORE_SELECT_POPUP_MODAL_UNDERLAY = ZO_SelectTradingHouseGuildDialogModalUnderlay


--===================== FUNCTIONS ==============================================
local function FCOGuildBankQuickSelect_SelectGuildById(guildId)
--d("FCOGuildBankQuickSelect_SelectGuildById: " ..guildId)
    -- Check to make sure were not trying to select a guildId that doesn't exist:
    local numGuilds = GetNumGuilds()
    if numGuilds == nil or numGuilds < 1 or guildId > numGuilds or guildId < 1 then return end

    if GUILDBANK_SELECT_POPUP_GUILD_COMBOBOX ~= nil and not GUILDBANK_SELECT_POPUP_GUILD_COMBOBOX.m_container:IsHidden() then
	    GUILDBANK_SELECT_POPUP_GUILD_COMBOBOX:SelectItemByIndex(guildId)
    end
    if GUILDSTORE_SELECT_POPUP_GUILD_COMBOBOX ~= nil and not GUILDSTORE_SELECT_POPUP_GUILD_COMBOBOX.m_container:IsHidden() then
	    GUILDSTORE_SELECT_POPUP_GUILD_COMBOBOX:SelectItemByIndex(guildId)
	end
end

--Select the next entry in the guild selection combobox
local function FCOGuildBankQuickSelect_ChangeGuildEntryByWheel(delta)
	local selectedItemData
    if GUILDBANK_SELECT_POPUP_GUILD_COMBOBOX ~= nil and not GUILDBANK_SELECT_POPUP_GUILD_COMBOBOX.m_container:IsHidden() then
		selectedItemData = GUILDBANK_SELECT_POPUP_GUILD_COMBOBOX.m_selectedItemData
    end
    if GUILDSTORE_SELECT_POPUP_GUILD_COMBOBOX ~= nil and not GUILDSTORE_SELECT_POPUP_GUILD_COMBOBOX.m_container:IsHidden() then
		selectedItemData = GUILDSTORE_SELECT_POPUP_GUILD_COMBOBOX.m_selectedItemData
	end

    -- The game doesn't initialize a default selectedItemData, so if one doesn't exist
    -- We'll just use 1 for the first item (which is what is shown by default)
    local selectedId = 1
    -- If it does exist, we'll overwrite that & use the selected guildId
    if selectedItemData then
        selectedId = selectedItemData.guildId
    end

    local numGuilds = GetNumGuilds()
    local newIndex = (selectedId - 1  - delta) % numGuilds + 1
    FCOGuildBankQuickSelect_SelectGuildById(newIndex)
end


-- set handlers for keyboard events:
local function FCOGuildBankQuickSelect_HandleKeyboardEvents(self, key, ctrl, alt, shift, command)
--d("GetKeyName(key): " .. GetKeyName(key) .. ", key: " .. key)
    local numGuilds = GetNumGuilds()
    if GetKeyName(key) == DIALOG_PRIMARY then
        ZO_Dialogs_ButtonKeybindPressed("DIALOG_PRIMARY")
    elseif GetKeyName(key) == DIALOG_NEGATIVE or key == KEY_ESCAPE then
        ZO_Dialogs_ButtonKeybindPressed("DIALOG_NEGATIVE")
    elseif key == KEY_1 or key == KEY_NUMPAD1 then
        FCOGuildBankQuickSelect_SelectGuildById(1)
    elseif key == KEY_2 or key == KEY_NUMPAD2 then
        FCOGuildBankQuickSelect_SelectGuildById(2)
    elseif key == KEY_3 or key == KEY_NUMPAD3 then
        FCOGuildBankQuickSelect_SelectGuildById(3)
    elseif key == KEY_4 or key == KEY_NUMPAD4 then
        FCOGuildBankQuickSelect_SelectGuildById(4)
    elseif key == KEY_5 or key == KEY_NUMPAD5 then
        FCOGuildBankQuickSelect_SelectGuildById(5)
    end
end

--==============================================================================
--==================  Register for keyboard/mouse events   =====================
--==============================================================================
local function FCOGuildBankQuickSelect_RegisterForMouseWheelEvents()
    GUILDBANK_SELECT_POPUP_MODAL_UNDERLAY:SetMouseEnabled(true)
    GUILDBANK_SELECT_POPUP_MODAL_UNDERLAY:SetHandler("OnMouseWheel", function(self, delta, ctrl, alt, shift)
        FCOGuildBankQuickSelect_ChangeGuildEntryByWheel(delta)
    end)

    GUILDSTORE_SELECT_POPUP_MODAL_UNDERLAY:SetMouseEnabled(true)
    GUILDSTORE_SELECT_POPUP_MODAL_UNDERLAY:SetHandler("OnMouseWheel", function(self, delta, ctrl, alt, shift)
        FCOGuildBankQuickSelect_ChangeGuildEntryByWheel(delta)
    end)
end

local function FCOGuildBankQuickSelect_ChangeTweaks(doRegister)
    GUILDBANK_SELECT_POPUP:SetKeyboardEnabled(true)
    --Register the OnKeyUp function for the guild bank select dialog
    ZO_PreHookHandler(GUILDBANK_SELECT_POPUP, 'OnKeyUp',
        function(self, key, ctrl, alt, shift, command)
            FCOGuildBankQuickSelect_HandleKeyboardEvents(self, key, ctrl, alt, shift, command)
    end)

    GUILDSTORE_SELECT_POPUP:SetKeyboardEnabled(true)
    --Register the OnKeyUp function for the guild bank select dialog
    ZO_PreHookHandler(GUILDSTORE_SELECT_POPUP, 'OnKeyUp',
        function(self, key, ctrl, alt, shift, command)
            FCOGuildBankQuickSelect_HandleKeyboardEvents(self, key, ctrl, alt, shift, command)
    end)
end

--==============================================================================
--==================== START EVENT CALLBACK FUNCTIONS===========================
--==============================================================================
--Addon loads up
local function FCOGuildBankQuickSelect_Loaded(eventCode, addOnName)
    if(addOnName ~= addonVars.gAddonName) then
        return
    end
    --Unregister this event again so it isn't fired again
    EVENT_MANAGER:UnregisterForEvent(addonVars.gAddonName, EVENT_ADD_ON_LOADED)

    -- Grabs the key for the binding string name (e, alt, whatever)
    DIALOG_PRIMARY = ZO_Keybindings_GetBindingStringFromAction("DIALOG_PRIMARY")
    DIALOG_NEGATIVE = ZO_Keybindings_GetBindingStringFromAction("DIALOG_NEGATIVE")

    FCOGuildBankQuickSelect_ChangeTweaks()
    FCOGuildBankQuickSelect_RegisterForMouseWheelEvents()
end


--------------------------------------------------------------------------------
--- Call the start function for this addon to register events etc.
--------------------------------------------------------------------------------
EVENT_MANAGER:RegisterForEvent(addonVars.gAddonName, EVENT_ADD_ON_LOADED, FCOGuildBankQuickSelect_Loaded) 
